/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package wssl.parser;
import com.parctechnologies.eclipse.CompoundTerm;
import java.io.IOException;
import java.net.URL;
import java.nio.file.Path;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Interface for WSSL parsers
 * @author George Baryannis
 * 
 */
public interface Parser {
    
    /**
     * Parses and validates a WSSL file. 
     * @param wsslfile {@link URL} pointing to the WSSL file to be parsed
     * @param schema {@link URL} containing the XML Schema to validate against
     * @return true if parsing and validation were successful
     * @throws ParserConfigurationException
     * @throws SAXException
     * @throws IOException
     */
    public boolean parse(URL wsslfile, URL schema)  throws ParserConfigurationException, SAXException, IOException;
    
    /**
     * Returns the set of WSSL inputs contained in the parsed document.
     * @return a {@link NodeList} containing WSSL input nodes
     */
    public NodeList getInp();
    
    /**
     * Returns the set of WSSL outputs contained in the parsed document.
     * @return a {@link NodeList} containing WSSL output nodes
     */    
    public NodeList getOutp();
    
    /**
     * Returns the set of WSSL preconditions contained in the parsed document.
     * @return a {@link NodeList} containing WSSL precondition nodes
     */    
    public NodeList getPre();
    
    /**
     * Returns the set of WSSL postconditions contained in the parsed document.
     * @return a {@link NodeList} containing WSSL postcondition nodes
     */        
    public NodeList getPost();
    
    /**
     * Returns the set of WSSL causal relationships contained in the parsed document.
     * @return a {@link NodeList} containing WSSL causal relationship nodes
     */        
    public NodeList getCR();
    
    /**
     * Returns the set of WSSL goals contained in the parsed document.
     * @return a {@link NodeList} containing WSSL goal nodes
     */        
    public NodeList getGoals();
    
    /**
     * Returns the set of QoS profiles contained in the parsed document.
     * @return a {@link NodeList} containing WSSL QoS profiles
     */        
    public NodeList getQoSProfiles();    
    
    /**
     * Returns the set of QoS constraints contained in the QoS Profile given as parameter
     * @param profileNode a {@link Node} containing a QoS Profile node
     * @return a {@link NodeList} containing WSSL QoS constraints
     */        
    public NodeList getQoSConstraints(Node profileNode); 
    
    /**
     * Returns the operator involved in the constraint passed as parameter.
     * @param constraintNode {@link Node} containing a WSSL QoS constraint
     * @return a {@link String} containing the operator name
     */
    public String getQoSOperator(Node constraintNode);
    
    /**
     * Returns the QoS attribute name associated with the constraint passes as parameter.
     * @param constraintNode {@link Node} containing a WSSL QoS constraint* 
     * @return a {@link String} containing the attribute name
     */
    public String getQoSAttribute(Node constraintNode);
    
    /**
     * Returns the QoS attribute value associated with the constraint passed as parameter.
     * @param constraintNode {@link Node} containing a WSSL QoS constraint
     * @return a the attribute value stored as a float.
     */
    public float getQoSValue(Node constraintNode);    
    
    /**
     * Returns the service information contained in the parsed document.
     * @return a {@link Node} corresponding to the WSSL service element
     */        
    public Node getService();    
    
    /**
     * Produces FLEX clauses for the given WSSL inputs and preconditions.
     * @param inpList a {@link NodeList} containing WSSL input nodes
     * @param preList a {@link NodeList} containing WSSL precondition nodes
     * @param numOut the number of service outputs (necessary, to solve the singleton problem)
     * @param path a {@link Path} pointing to the FLEX file where the clauses are written
     * @throws InvalidFormatException
     * @throws IOException
     */
    public void writePre(NodeList inpList, NodeList preList, int numOut, Path path) throws InvalidFormatException, IOException;
    
    /**
     * Produces FLEX clauses for the given WSSL outputs and postconditions.
     * @param outpList a {@link NodeList} containing WSSL output nodes
     * @param postList a {@link NodeList} containing WSSL postcondition nodes
     * @param path a {@link Path} pointing to the FLEX file where the clauses are written
     * @throws InvalidFormatException
     * @throws IOException
     */    
    public void writePost(NodeList outpList, NodeList postList, Path path) throws InvalidFormatException, IOException;
    
    /**
     * Produces FLEX clauses for the given WSSL causal relationships.
     * @param crList a {@link NodeList} containing WSSL causal relationship nodes
     * @param path a {@link Path} pointing to the FLEX file where the clauses are written
     * @throws InvalidFormatException
     * @throws IOException
     */     
    public void writeCR(NodeList crList, Path path)  throws InvalidFormatException, IOException;    
    
    /**
     * Produces FLEX clauses for the given WSSL goals.
     * @param goalList a {@link NodeList} containing WSSL goals
     * @return the FLEX goals produced
     * @throws InvalidFormatException
     */
    public String writeGoals(NodeList goalList) throws InvalidFormatException;
    
    /**
     * Produces FLEX clauses for the given WSSL goals.
     * @param goalList a {@link NodeList} containing WSSL goals
     * @return the FLEX goals as a {@link CompoundTerm} array object
     * @throws InvalidFormatException
     */
    public CompoundTerm[] writeGoalsCT(NodeList goalList) throws InvalidFormatException;    
}
